'''PMD1608Wrap.py - A Python Interface to MCC PMD-1608FS Device Library.
   =====================================================================

   Access most functions of the Measurement Computing PMD-1608FS USB data
   acquisition and digital control device.

   Author: Nick Glazzard 2013'''

from ctypes import *
from numpy import *
from time import sleep

lib = cdll.LoadLibrary( './PMD1608lib.so' )

def argerr():
    raise RuntimeError( 'Invalid arguments.' )

def callerr():
    raise RuntimeError( 'PMD1608lib call failed.' )

class PMD1608(object):
    '''Class that encapsulates PMD-1608FS functionality.'''
    def __init__(self):
        self.obj = lib.PMD1608FS_new( c_int(0) )

    def set_byte_in( self ):
        '''Set all 8 DIO lines to be inputs.'''
        if( lib.PMD1608FS_set_byte_in(self.obj) != 1 ):
            callerr()
        return True

    def set_byte_out( self ):
        '''Set all 8 DIO lines to be outputs.'''
        if( lib.PMD1608FS_set_byte_out(self.obj) != 1 ):
            callerr()
        return True

    def set_bit_in( self, bitnum ):
        '''Set DIO line bitnum to be an input.'''
        if( lib.PMD1608FS_set_bit_in(self.obj, c_int(bitnum) ) != 1 ):
            callerr()
        return True

    def set_bit_out( self, bitnum ):
        '''Set DIO line bitnum to be an output.'''
        if( lib.PMD1608FS_set_bit_out(self.obj, c_int(bitnum) ) != 1 ):
            callerr()
        return True

    def read_byte( self, byte ):
        '''Read all 8 DIO lines as a byte value.'''
        inbyte = c_ubyte(0)
        if( lib.PMD1608FS_read_byte(self.obj, byref(inbyte) ) != 1 ):
            callerr()
        return( inbyte.value )

    def write_byte( self, byte ):
        '''Set all 8 DIO lines from a byte value.'''
        if( lib.PMD1608FS_write_byte(self.obj, c_ubyte(byte) ) != 1 ):
            callerr()
        return True
         
    def read_bit( self, bitnum ):
        '''Read the state of DIO line bitnum.'''
        inbit = c_int(0)
        if( lib.PMD1608FS_read_bit( self.obj, c_int(bitnum), byref(inbit) ) != 1 ):
            callerr()
        return( inbit.value )

    def write_bit( self, bitnum, bitval ):
        '''Set the state of DIO line bitnum to bitval (0 or 1).'''
        if( lib.PMD1608FS_write_bit( self.obj, c_int(bitnum), c_int(bitval) ) != 1 ):
            callerr()
        return True

    def pulse_bit( self, bitnum, usecs ):
        '''Set the state of DIO line bitnum to 1 for usecs microseconds then 0.
        N.B. Minimum achievable pulse width is about 3ms.'''
        if( lib.PMD1608FS_pulse_bit( self.obj, c_int(bitnum), c_int(usecs) ) != 1 ):
            callerr()
        return True

    def read_channel( self, channel, grange ):
        '''Read the voltage on analog input channel using full scale range grange.
        grange may be: '10V', '5V', '2.5V', '2V', '1.25V', '1V', '625mV', '312mV' '''
        gns = { '10V':0, '5V':1, '2.5V':2, '2V':3, '1.25V':4, '1V':5, '625mV':6, '312mV':7,
                '10v':0, '5v':1, '2.5v':2, '2v':3, '1.25v':4, '1v':5, '625mv':6, '312mv':7 }
        try:
            igrange = gns[grange]
        except Exception, err:
            argerr()
        lib.PMD1608FS_read_channel.restype = c_float
        return lib.PMD1608FS_read_channel( self.obj, c_int(channel), c_int(igrange) )

    def reset_counter( self ):
        '''Reset the counter.'''
        if( lib.PMD1608FS_reset_counter( self.obj ) != 1 ):
            callerr()
        return True

    def read_counter( self ):
        '''Read the counter value.'''
        incount = c_int(0)
        if( lib.PMD1608FS_read_counter( self.obj, byref(incount) ) != 1 ):
            callerr()
        return( incount.value )

    def waitms( self, ms ):
        '''Wait some milliseconds.'''
        sleep( 0.001*ms )


class Stepper(object):
    '''Control a 2-Y stepper motor via PMD1608.
    This is for a Muirhead Vactric 23MR302 motor controlled via the ANGEL
    stepper motor driver.'''
    def __init__(self):
        self.pmd = PMD1608()
        self.pmd.set_byte_out()
        self.pmd.write_byte( 0 )
        self.step = 0
        self.waitfor = 2
        self.lastdirection = 1
        self.deg_per_step= 1.8 # Degrees per full step.
        self.angle = 0.0
        self.mode = 1 # Full step mode.
        self.full_traj = [12,9,3,6] # Bit order: x1,x2,y1,y2
        self.half_traj = [12,8,9,1,3,2,6,4]

    def reset_angle(self):
        '''Set current angle as zero angle.'''
        self.angle = 0.0

    def get_angle(self):
        '''Return the current angle achieved.'''
        return self.angle

    def current_off(self):
        '''Turn off all current to motor.'''
        self.pmd.write_byte( 0 )

    def set_mode(self,mode):
        '''Set full or half step mode.'''
        if( mode == 'full'):
            self.mode = 1
            self.deg_per_step = 1.8
        else:
            self.mode = 2
            self.deg_per_step = 0.9

    def set_wait(self,wait):
        '''Set milliseconds to wait after every step. 
        Limits acceleration (sort of).'''
        if( wait < 2 ):
            self.waitfor = 2
        else:
            self.waitfor = wait

    def step_forward(self):
        '''Step forward (clockwise) one step.'''
        if( self.mode == 1 ):
            k = self.step % 4
            self.pmd.write_byte( self.full_traj[k] )
        else:
            k = self.step % 8
            self.pmd.write_byte( self.half_traj[k] )
        self.step += 1
        self.pmd.waitms( self.waitfor )
        self.angle += self.deg_per_step

    def step_backward(self):
        '''Step backwards (widdershins) one step.'''
        self.step -= 1
        if( self.mode == 1 ):
            k = self.step % 4
            self.pmd.write_byte( self.full_traj[k] )
        else:
            k = self.step % 8
            self.pmd.write_byte( self.half_traj[k] )
        self.pmd.waitms( self.waitfor )
        self.angle -= self.deg_per_step

    def steps(self,nsteps):
        '''Move nsteps forward (nsteps>0) or backwards (nsteps<0).'''
        if( nsteps == 0 ):
            return
        direction = cmp(nsteps,0)
        if( direction < self.lastdirection ):
            self.step -= 1
        elif( direction > self.lastdirection ):
            self.step += 1
        self.lastdirection = direction
        for i in range(0,nsteps,direction):
            if( direction > 0 ):
                self.step_forward()
            else:
                self.step_backward()

    def step_angle(self,angle):
        '''Move by angle (forwards or backwards).'''
        nsteps = int(round(angle/self.deg_per_step))
        self.steps(nsteps)

    def goto_angle(self,angle,hold=False):
        '''Go to a specified angle. Return the angular positioning error.
        If hold True, maintain current in the coils. In that case, the motors
        holding torque should be needed to turn the motor shaft with an external
        torque. Otherwise, turn off current to the coils, in which case the (lower)
        detent torque should suffice to turn the shaft.'''
        delta_angle = angle - self.angle
        self.step_angle(delta_angle)
        if( not hold):
            self.current_off()
        return( self.angle - angle )

    def waitms(self,waitfor):
        self.pmd.waitms(waitfor)


if __name__ == '__main__':
    if( False ):
        d = PMD1608()
        for i in range(0,8):
            print d.read_channel( i, '1V' )
        d.set_bit_out( 0 )
        d.write_bit( 0, 0 )
        d.reset_counter()
        for i in range(0,100):
            d.write_bit( 0, 1 )
            d.waitms( 100 )
            d.write_bit( 0, 0 )
            print d.read_counter()

    if( True ):
        s = Stepper()
        s.set_mode( 'full' )
        #s.step_forward()
        for j in range(0,1):
            for i in range(0,10):
                s.steps(10)
                s.steps(-10)
            s.step_angle(180)
            s.step_angle(-180)
            aerr = s.goto_angle(37.5)
            print 'Go to 37.5, error=', aerr, ' angle=', s.get_angle()
            s.goto_angle(90)
            s.waitms(500)
            s.goto_angle(180)
            s.waitms(500)
            s.goto_angle(270)
            s.waitms(500)
            s.goto_angle(360)
            s.waitms(500)
            s.goto_angle(270)
            s.waitms(500)
            s.goto_angle(180)
            s.waitms(500)
            s.goto_angle(90)
            s.waitms(500)
            s.goto_angle(0)
        #s.current_off()
