#include "pmd1608FS.h"

PMD1608FS::PMD1608FS( bool calibrate )
//------------------------------------
// Find a PMD-1608FS and open it. Read its calibration table (unless calibrate true).
{
  opened = false;
  fd = -1;

  // Find the thing.
  int nInterfaces = PMD_Find(MCC_VID, USB1608FS_PID, &fd, 1);
  if (nInterfaces <= 0) {
    fprintf(stderr, "USB 1608FS not found.\n");
    return;
  }

  // Get its serial number.
  if( usbReadSerial_USB1608FS( fd, serial ) < 0 ){
    fprintf(stderr, "usbReadSerial() failed.\n");
    return;
  }

  // Make up a calibration file name.
  sprintf( calfile, "/usr/local/pmd1608fs/caltab_%s.cal", serial );

  // Read its calibration table.
  if( ! calibrate ){
    if( ! usbCalibrationIO( caltab, calfile, 0 ) ){
      fprintf(stderr, "usbCalibrationIO() failed.\n");
      return;
    }
  }

  // Success. Maybe.
  opened = true;
}

PMD1608FS::~PMD1608FS()
{
  if( fd != -1 ){
    close( fd );
  }
}

//***************
//* CALIBRATION *
//***************

bool PMD1608FS::calibrate( float* known_calvolts, int slowly )
//------------------------------------------------------------
// Create a calibration table. These are specific to the device
// which is identified by its serial number. If known_calvolts is
// not NULL, it should be a float[5] array containing the measured
// voltage at the cal terminal for each calibration level. If slowly
// is set true, the process is done slowly enough to measure these voltages.
// So: calibrate( NULL, 1 ), to read the voltages, then
//     calibrate( measured_volts, 0 )
// is the full process for best accuracy.
// Calibration tables are stored in: /usr/local/pmd1608fs/
// which must exist and be writeable.
{
  if( opened ){
    if( ! usbCalibrate_USB1608FS( fd, caltab, slowly, known_calvolts ) )
      return( false );
    if( ! usbCalibrationIO( caltab, calfile, 1 ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

//***************
//* DIGITAL I/O *
//***************

bool PMD1608FS::set_byte_in()
//---------------------------
// Set all 8 DIO lines to be inputs.
{
  if( opened ){
    if( ! usbDConfigPort_USB1608FS( fd, DIO_DIR_IN ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::set_byte_out()
//----------------------------
// Set all 8 DIO lines to be outputs.
{
  if( opened ){
    if( ! usbDConfigPort_USB1608FS( fd, DIO_DIR_OUT ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::set_bit_in( int bitnum )
//--------------------------------------
// Set DIO line bitnum to be an input.
{
  if( bitnum < 0 || bitnum > 7 )
    return( false );
  if( opened ){
    if( ! usbDConfigBit_USB1608FS( fd, bitnum, DIO_DIR_IN ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::set_bit_out( int bitnum )
//---------------------------------------
// Set DIO line bitnum to be an output.
{
  if( bitnum < 0 || bitnum > 7 )
    return( false );
  if( opened ){
    if( ! usbDConfigBit_USB1608FS( fd, bitnum, DIO_DIR_OUT ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::read_byte( unsigned char& byte )
//----------------------------------------------
// Read the state of all 8 DIO lines in to byte.
{
  unsigned char inbyte;
  if( opened ){
    if( ! usbDIn_USB1608FS( fd, &inbyte ) )
      return( false );
    byte = inbyte;
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::write_byte( unsigned char byte )
//----------------------------------------------
// Set the state of all 8 DIO lines from byte.
{
  if( opened ){
    if( ! usbDOut_USB1608FS( fd, byte ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::read_bit( int bitnum, int& bitval )
//-------------------------------------------------
// Read the state of DIO line bitnum in to bitval (0 or 1).
{
  if( bitnum < 0 || bitnum > 7 )
    return( false );
  unsigned char cbitnum, cbitval;
  cbitnum = (unsigned char)bitnum;
  if( opened ){
    if( ! usbDInBit_USB1608FS( fd, cbitnum, &cbitval) )
      return( false );
    bitval = cbitval ? 0 : 1 ;
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::write_bit( int bitnum, int bitval )
//-------------------------------------------------
// Set the state of DIO line bitnum from bitval.
{
  if( bitnum < 0 || bitnum > 7 )
    return( false );
  unsigned char cbitnum, cbitval;
  cbitnum = (unsigned char)bitnum;
  cbitval = bitval ? 0 : 1 ;
  if( opened ){
    if( ! usbDOutBit_USB1608FS( fd, cbitnum, cbitval) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::pulse_bit( int bitnum, int usecs )
//-------------------------------------------------
// Set the state of DIO line bitnum high for usecs microseconds, then low.
{
  if( bitnum < 0 || bitnum > 7 )
    return( false );
  unsigned char cbitnum;
  cbitnum = (unsigned char)bitnum;
  if( opened ){
    if( ! usbDOutBit_USB1608FS( fd, cbitnum, 1) )
      return( false );
    usleep( (useconds_t)usecs );
    if( ! usbDOutBit_USB1608FS( fd, cbitnum, 0) )
      return( false );
    return( true );
  }
  else
    return( false );
}


//****************
//* ANALOG INPUT *
//****************

bool PMD1608FS::read_channel( int channel, int range, float& value )
//------------------------------------------------------------------
// Read the voltage present on analog input channel, using range (full scale volts)
// returning the measurement in value.
{
  if( channel < 0 || channel > 7 )
    return( false );
  if( range < 0 || range > 7 )
    return( false );
  if( opened ){
    unsigned char cchan = (unsigned char)channel;
    unsigned char crange = (unsigned char)range;
    float involts = usbAIn_USB1608FS( fd, cchan, crange, caltab );
    value = involts;
    if( involts < -99999.0 )
      return( false );
    return( true );
  }
  else
    return( false );
}

//***********
//* COUNTER *
//***********

bool PMD1608FS::reset_counter()
//-----------------------------
// Reset the counter to zero.
{
  if( opened ){
    if( ! usbInitCounter_USB1608FS( fd ) )
      return( false );
    return( true );
  }
  else
    return( false );
}

bool PMD1608FS::read_counter( int& value )
//----------------------------------------
// Read the counter.
{
  if( opened ){
    unsigned int count = usbReadCounter_USB1608FS( fd );
    value = (count & 0x7fffffff);
    return( true );
  }
  else
    return( false );
}

// C bindings for Python Ctypes.
extern "C" {
  PMD1608FS* PMD1608FS_new( int calibrate )
  {return new PMD1608FS( calibrate!=0 );}
  int PMD1608FS_calibrate( PMD1608FS* s )
  {return(s->calibrate( NULL, 0 )?1:0);}
  int PMD1608FS_set_byte_in( PMD1608FS* s )
  {return(s->set_byte_in()?1:0);}
  int PMD1608FS_set_byte_out( PMD1608FS* s )
  {return(s->set_byte_out()?1:0);}
  int PMD1608FS_set_bit_in( PMD1608FS* s, int bitnum )
  {return(s->set_bit_in( bitnum )?1:0);}
  int PMD1608FS_set_bit_out( PMD1608FS* s, int bitnum )
  {return(s->set_bit_out( bitnum )?1:0);}
  int PMD1608FS_read_byte( PMD1608FS* s, unsigned char* pbyte )
  {unsigned char uc; int r=s->read_byte(uc)?1:0; *pbyte=uc; return r;}
  int PMD1608FS_write_byte( PMD1608FS* s, unsigned char byte )
  {return(s->write_byte(byte)?1:0);}
  int PMD1608FS_read_bit( PMD1608FS* s, int bitnum, int* bitval )
  {int ib; int r=s->read_bit( bitnum, ib )?1:0; *bitval=ib; return r;}
  int PMD1608FS_write_bit( PMD1608FS* s, int bitnum, int bitval )
  {return(s->write_bit(bitnum, bitval)?1:0);}
  int PMD1608FS_pulse_bit( PMD1608FS* s, int bitnum, int usecs )
  {return(s->pulse_bit(bitnum,usecs)?1:0);}
  float PMD1608FS_read_channel( PMD1608FS* s, int channel, int range )
  {float v; int r=s->read_channel(channel, range, v); return(r?v:-9999999.0f);}
  int PMD1608FS_reset_counter( PMD1608FS* s )
  {return(s->reset_counter()?1:0);}
  int PMD1608FS_read_counter( PMD1608FS* s, int* count )
  {int ic; int r=s->read_counter(ic)?1:0; *count=ic; return r;}
}
